package com.m2pool.common.core.Result;

import com.m2pool.common.core.constant.Constants;
import lombok.Data;

import java.io.Serializable;

/**
 * @Description 响应信息主体
 * @Date 2024/6/11 9:18
 * @Author dy
 */
@Data
public class R<T> implements Serializable
{
    private static final long serialVersionUID = 3276842119125241681L;

    /** 成功 */
    public static final int SUCCESS = Constants.SUCCESS;

    /** 失败 */
    public static final int FAIL = Constants.FAIL;

    private int code;

    private String msg;

    private T data;

    public static <T> R<T> success()
    {
        return restResult(null, SUCCESS, null);
    }

    public static <T> R<T> success(T data)
    {
        return restResult(data, SUCCESS, null);
    }

    public static <T> R<T> success(T data, String msg)
    {
        return restResult(data, SUCCESS, msg);
    }

    public static <T> R<T> fail()
    {
        return restResult(null, FAIL, null);
    }

    public static <T> R<T> fail(String msg)
    {
        return restResult(null, FAIL, msg);
    }

    public static <T> R<T> fail(T data)
    {
        return restResult(data, FAIL, null);
    }

    public static <T> R<T> fail(T data, String msg)
    {
        return restResult(data, FAIL, msg);
    }

    public static <T> R<T> fail(int code, String msg)
    {
        return restResult(null, code, msg);
    }

    private static <T> R<T> restResult(T data, int code, String msg)
    {
        R<T> apiResult = new R<>();
        apiResult.setCode(code);
        apiResult.setData(data);
        apiResult.setMsg(msg);
        return apiResult;
    }
}
