package com.m2pool.pool.task;

import com.m2pool.common.core.utils.DateUtils;
import com.m2pool.common.core.utils.StringUtils;
import com.m2pool.common.redis.service.RedisService;
import com.m2pool.pool.entity.BlockInfo;
import com.m2pool.pool.mapper.PoolMapper;
import com.m2pool.pool.utils.NodeRpc;
import com.m2pool.pool.vo.DateValueVo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import java.util.Date;
import java.util.Map;

/**
 * @Description TODO
 * @Date 2024/6/14 14:03
 * @Author dy
 */
@Configuration
@EnableScheduling
public class NodeTask {


    private static final Logger log = LoggerFactory.getLogger(NodeTask.class);

    @Autowired
    private PoolMapper poolMapper;

    @Autowired
    private RedisService redisService;

    //@Scheduled(cron = "0 0/2 * * * ?")
    @Scheduled(cron = "5 0,30 * * * ?")
    public void NEXABlockInfoToRedisAndDB(){
        BlockInfo blockInfo = NodeRpc.getBlock("nexa");

        int  count = 1;
        while (StringUtils.isNull(blockInfo)){
            if(count >= 10){
                break;
            }
            blockInfo = NodeRpc.getBlock("nexa");
            count++;
        }

        if(StringUtils.isNotNull(blockInfo)){
            if(StringUtils.isNotNull(blockInfo.getPower())){
                redisService.setCacheObject("nexa_block",blockInfo);
                //存入数据库
                DateValueVo vo = new DateValueVo();
                Date now = new Date();
                int minute = (now.getMinutes() / 30) * 30;
                now.setMinutes( minute);
                now.setSeconds(0);
                vo.setDate(now);
                String date = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, now);
                System.out.println("nexa全网算力数据入库时间:"+date);
                vo.setValue(blockInfo.getPower());
                poolMapper.insertNetPower("nexa_net_power",date,vo);
            }
        }
    }

    @Scheduled(cron = "0 0/1 * * * ?")
    public void NEXABlockInfoToRedis(){
        BlockInfo blockInfo = NodeRpc.getBlock("nexa");

        int  count = 0;
        while (StringUtils.isNull(blockInfo)){
            if(count >= 3){
                break;
            }
            blockInfo = NodeRpc.getBlock("nexa");
            count++;
        }

        if(StringUtils.isNotNull(blockInfo)){
            if(StringUtils.isNotNull(blockInfo.getPower())){

                if (redisService.hasKey("nexa_block")){
                    redisService.deleteObject("nexa_block");
                    redisService.setCacheObject("nexa_block",blockInfo);
                }else {
                    redisService.setCacheObject("nexa_block",blockInfo);
                }

            }
        }
    }

    @Scheduled(cron = "10 0,30 * * * ?")
    public void GRSBlockInfoToRedisAndDB(){
        BlockInfo blockInfo = NodeRpc.getBlock("grs");

        int  count = 1;
        while (StringUtils.isNull(blockInfo)){
            if(count >= 10){
                break;
            }
            blockInfo = NodeRpc.getBlock("grs");
            count++;
        }

        if(StringUtils.isNotNull(blockInfo)){
            if(StringUtils.isNotNull(blockInfo.getPower())){
                redisService.setCacheObject("grs_block",blockInfo);
                //存入数据库
                DateValueVo vo = new DateValueVo();
                Date now = new Date();
                int minute = (now.getMinutes() / 30) * 30;
                now.setMinutes( minute);
                now.setSeconds(0);
                vo.setDate(now);
                String date = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, now);
                vo.setValue(blockInfo.getPower());
                poolMapper.insertNetPower("grs_net_power",date,vo);
            }
        }
    }

    @Scheduled(cron = "2 0/1 * * * ?")
    public void GRSBlockInfoToRedis(){
        BlockInfo blockInfo = NodeRpc.getBlock("grs");

        if(StringUtils.isNotNull(blockInfo)){
            if(StringUtils.isNotNull(blockInfo.getPower())){

                if (redisService.hasKey("grs_block")){
                    redisService.deleteObject("grs_block");
                    redisService.setCacheObject("grs_block",blockInfo);
                }else {
                    redisService.setCacheObject("grs_block",blockInfo);
                }

            }
        }
        else {
            System.out.println("未从节点获取到grs最新数据 ："+DateUtils.dateTimeNow());
        }
    }

    @Scheduled(cron = "15 0,30 * * * ?")
    public void MONABlockInfoToRedisAndDB(){
        BlockInfo blockInfo = NodeRpc.getBlock("mona");

        int  count = 1;
        while (StringUtils.isNull(blockInfo)){
            if(count >= 10){
                break;
            }
            blockInfo = NodeRpc.getBlock("mona");
            count++;
        }

        if(StringUtils.isNotNull(blockInfo)){
            if(StringUtils.isNotNull(blockInfo.getPower())){
                redisService.setCacheObject("mona_block",blockInfo);
                //存入数据库
                DateValueVo vo = new DateValueVo();
                Date now = new Date();
                int minute = (now.getMinutes() / 30) * 30;
                now.setMinutes( minute);
                now.setSeconds(0);
                vo.setDate(now);
                String date = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, now);
                System.out.println("mona 全网算力数据入库时间:"+date);
                vo.setValue(blockInfo.getPower());
                poolMapper.insertNetPower("mona_net_power",date,vo);
            }
        }
    }

    @Scheduled(cron = "4 0/1 * * * ?")
    public void MONABlockInfoToRedis(){
        BlockInfo blockInfo = NodeRpc.getBlock("mona");

        if(StringUtils.isNotNull(blockInfo)){
            if(StringUtils.isNotNull(blockInfo.getPower())){

                if (redisService.hasKey("mona_block")){
                    redisService.deleteObject("mona_block");
                    redisService.setCacheObject("mona_block",blockInfo);
                }else {
                    redisService.setCacheObject("mona_block",blockInfo);
                }

            }
        }else {
            System.out.println("未从节点获取到mona最新数据 ："+DateUtils.dateTimeNow());
        }

    }

    @Scheduled(cron = "20 0,30 * * * ?")
    public void DGBBlockInfoToRedisAndDB(){
        Map<String, BlockInfo> dgbMap = NodeRpc.getDGBBlock();
        int  count = 1;
        while (StringUtils.isNull(dgbMap)){
            if(count >= 10){
                break;
            }
            dgbMap = NodeRpc.getDGBBlock();
            count++;
        }
        if(StringUtils.isNotNull(dgbMap)){
            BlockInfo dgbo = dgbMap.getOrDefault("dgbo", null);
            if(StringUtils.isNotNull(dgbo)){
                if(StringUtils.isNotNull(dgbo.getPower())){
                    redisService.setCacheObject("dgbo_block",dgbo);
                    //存入数据库
                    DateValueVo vo = new DateValueVo();
                    Date now = new Date();
                    int minute = (now.getMinutes() / 30) * 30;
                    now.setMinutes( minute);
                    now.setSeconds(0);
                    vo.setDate(now);
                    String date = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, now);
                    System.out.println("dgbo 全网算力数据入库时间:"+date);
                    vo.setValue(dgbo.getPower());
                    poolMapper.insertNetPower("dgbo_net_power",date,vo);
                }else {
                    System.out.println("未获取到dgbo的power");
                }
            }
            else {
                System.out.println("未获取到dgbo");
            }

            BlockInfo dgbq = dgbMap.getOrDefault("dgbq", null);
            if(StringUtils.isNotNull(dgbq)){
                if(StringUtils.isNotNull(dgbo.getPower())){
                    redisService.setCacheObject("dgbq_block",dgbq);
                    //存入数据库
                    DateValueVo vo = new DateValueVo();
                    Date now = new Date();
                    int minute = (now.getMinutes() / 30) * 30;
                    now.setMinutes( minute);
                    now.setSeconds(0);
                    vo.setDate(now);
                    String date = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, now);
                    System.out.println("dgbq 全网算力数据入库时间:"+date);
                    vo.setValue(dgbq.getPower());
                    poolMapper.insertNetPower("dgbq_net_power",date,vo);
                }
                else {
                    System.out.println("未获取到dgbo的power");
                }
            }
            else {
                System.out.println("未获取到dgbq");
            }

            BlockInfo dgbs = dgbMap.getOrDefault("dgbs", null);
            if(StringUtils.isNotNull(dgbs)){
                if(StringUtils.isNotNull(dgbs.getPower())){
                    redisService.setCacheObject("dgbs_block",dgbs);
                    //存入数据库
                    DateValueVo vo = new DateValueVo();
                    Date now = new Date();
                    int minute = (now.getMinutes() / 30) * 30;
                    now.setMinutes( minute);
                    now.setSeconds(0);
                    vo.setDate(now);
                    String date = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, now);
                    System.out.println("dgbs 全网算力数据入库时间:"+date);
                    vo.setValue(dgbs.getPower());
                    poolMapper.insertNetPower("dgbs_net_power",date,vo);
                }
                else {
                    System.out.println("未获取到dgbs的power");
                }
            }else {
                System.out.println("未获取到dgbs");
            }

        }else {
            System.out.println("未从节点获取到dgpMap最新数据 ："+DateUtils.dateTimeNow());
        }
    }

    @Scheduled(cron = "6 0/1 * * * ?")
    public void DGBBlockInfoToRedis(){
        Map<String, BlockInfo> dgbMap = NodeRpc.getDGBBlock();
        if(StringUtils.isNotNull(dgbMap)){
            BlockInfo dgbo = dgbMap.getOrDefault("dgbo", null);
            if(StringUtils.isNotNull(dgbo)){

                if (redisService.hasKey("dgbo_block")){
                    redisService.deleteObject("dgbo_block");
                    redisService.setCacheObject("dgbo_block",dgbo);
                }else {
                    redisService.setCacheObject("dgbo_block",dgbo);
                }

            }
            BlockInfo dgbq = dgbMap.getOrDefault("dgbq", null);
            if(StringUtils.isNotNull(dgbq)){

                if (redisService.hasKey("dgbq_block")){
                    redisService.deleteObject("dgbq_block");
                    redisService.setCacheObject("dgbq_block",dgbq);
                }else {
                    redisService.setCacheObject("dgbq_block",dgbq);
                }

            }
            BlockInfo dgbs = dgbMap.getOrDefault("dgbs", null);
            if(StringUtils.isNotNull(dgbs)){

                if (redisService.hasKey("dgbs_block")){
                    redisService.deleteObject("dgbs_block");
                    redisService.setCacheObject("dgbs_block",dgbs);
                }else {
                    redisService.setCacheObject("dgbs_block",dgbs);
                }

            }

        }else {
            System.out.println("未从节点获取到dgpMap最新数据 ："+DateUtils.dateTimeNow());
        }

    }


    @Scheduled(cron = "25 0,30 * * * ?")
    public void RXDBlockInfoToRedisAndDB(){
        BlockInfo blockInfo = NodeRpc.getBlock("rxd");

        int  count = 1;
        while (StringUtils.isNull(blockInfo)){
            if(count >= 10){
                break;
            }
            blockInfo = NodeRpc.getBlock("rxd");
            count++;
        }

        if(StringUtils.isNotNull(blockInfo)){
            if(StringUtils.isNotNull(blockInfo.getPower())){
                redisService.setCacheObject("rxd_block",blockInfo);
                //存入数据库
                DateValueVo vo = new DateValueVo();
                Date now = new Date();
                int minute = (now.getMinutes() / 30) * 30;
                now.setMinutes( minute);
                now.setSeconds(0);
                vo.setDate(now);
                String date = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, now);
                System.out.println("rxd 全网算力数据入库时间:"+date);
                vo.setValue(blockInfo.getPower());
                poolMapper.insertNetPower("rxd_net_power",date,vo);
            }
        }
    }

    @Scheduled(cron = "8 0/1 * * * ?")
    public void RXDBlockInfoToRedis(){
        BlockInfo blockInfo = NodeRpc.getBlock("rxd");

        if(StringUtils.isNotNull(blockInfo)){
            if(StringUtils.isNotNull(blockInfo.getPower())){

                if (redisService.hasKey("rxd_block")){
                    redisService.deleteObject("rxd_block");
                    redisService.setCacheObject("rxd_block",blockInfo);
                }else {
                    redisService.setCacheObject("rxd_block",blockInfo);
                }

            }
        }else {
            System.out.println("未从节点获取到最新数据 ："+DateUtils.dateTimeNow());
        }

    }

    @Scheduled(cron = "30 0,30 * * * ?")
    public void ALPHBlockInfoToRedisAndDB(){
        BlockInfo blockInfo = NodeRpc.getBlock("alph");
        //改成根据swagger api调用 用与原有节点调用相比新的路径、新的api

        int  count = 1;
        while (StringUtils.isNull(blockInfo)){
            if(count >= 10){
                break;
            }
            blockInfo = NodeRpc.getBlock("alph");
            count++;
        }

        if(StringUtils.isNotNull(blockInfo)){
            if(StringUtils.isNotNull(blockInfo.getPower())){
                redisService.setCacheObject("alph_block",blockInfo);
                //存入数据库
                DateValueVo vo = new DateValueVo();
                Date now = new Date();
                int minute = (now.getMinutes() / 30) * 30;
                now.setMinutes( minute);
                now.setSeconds(0);
                vo.setDate(now);
                String date = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, now);
                System.out.println("alph 全网算力数据入库时间:"+date);
                vo.setValue(blockInfo.getPower());
                poolMapper.insertNetPower("alph_net_power",date,vo);
            }
        }
    }

    @Scheduled(cron = "14 0/1 * * * ?")
    public void ALPHBlockInfoToRedis(){
        BlockInfo blockInfo = NodeRpc.getBlock("alph");

        if(StringUtils.isNotNull(blockInfo)){
            if(StringUtils.isNotNull(blockInfo.getPower())){

                if (redisService.hasKey("alph_block")){
                    redisService.deleteObject("alph_block");
                    redisService.setCacheObject("alph_block",blockInfo);
                }else {
                    redisService.setCacheObject("alph_block",blockInfo);
                }

            }
        }else {
            System.out.println("alph 未从节点获取到最新数据 ："+DateUtils.dateTimeNow());
        }

    }

    @Scheduled(cron = "35 0,30 * * * ?")
    public void ENXBlockInfoToRedisAndDB(){
        BlockInfo blockInfo = NodeRpc.getBlock("enx");

        int  count = 1;
        while (StringUtils.isNull(blockInfo)){
            if(count >= 10){
                break;
            }
            blockInfo = NodeRpc.getBlock("enx");
            count++;
        }

        if(StringUtils.isNotNull(blockInfo)){
            if(StringUtils.isNotNull(blockInfo.getPower())){
                redisService.setCacheObject("enx_block",blockInfo);
                //存入数据库
                DateValueVo vo = new DateValueVo();
                Date now = new Date();
                int minute = (now.getMinutes() / 30) * 30;
                now.setMinutes( minute);
                now.setSeconds(0);
                vo.setDate(now);
                String date = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, now);
                System.out.println("enx 全网算力数据入库时间:"+date);
                vo.setValue(blockInfo.getPower());
                poolMapper.insertNetPower("enx_net_power",date,vo);
            }
        }
    }

    @Scheduled(cron = "12 0/1 * * * ?")
    public void ENXBlockInfoToRedis(){
        BlockInfo blockInfo = NodeRpc.getBlock("enx");

        if(StringUtils.isNotNull(blockInfo)){
            if(StringUtils.isNotNull(blockInfo.getPower())){

                if (redisService.hasKey("enx_block")){
                    redisService.deleteObject("enx_block");
                    redisService.setCacheObject("enx_block",blockInfo);
                }else {
                    redisService.setCacheObject("enx_block",blockInfo);
                }

            }
        }else {
            System.out.println("未从节点获取到最新数据 ："+DateUtils.dateTimeNow());
        }

    }


    /**
     * 全网报块高度入库
     */
    @Scheduled(cron = "0 0 0 * * ?")
    @Async("customTaskThreadPool")
    public void insertNetBlock(){
        insertBlockWithRetry("nexa", "nexa_net_block", 10);
        insertBlockWithRetry("dgbq", "dgbq_net_block", 10);
        insertBlockWithRetry("dgbo", "dgbo_net_block", 10);
        insertBlockWithRetry("dgbs", "dgbs_net_block", 10);
        insertBlockWithRetry("mona", "mona_net_block", 10);
        insertBlockWithRetry("rxd", "rxd_net_block", 10);
        insertBlockWithRetry("grs", "grs_net_block", 10);
    }

    private void insertBlockWithRetry(String blockName, String tableName, int maxRetries) {
        int retryCount = 0;
        while (retryCount < maxRetries) {
            try {
                BlockInfo block = NodeRpc.getBlock(blockName);
                poolMapper.insertNetBlock(tableName, block.getHeight());
                return;
            } catch (Exception e) {
                retryCount++;
                if (retryCount >= maxRetries) {
                    log.error("插入区块 {} 数据失败，重试 {} 次后仍失败", blockName, maxRetries, e);
                    break;
                }
                try {
                    Thread.sleep(500);
                } catch (InterruptedException ie) {
                    Thread.currentThread().interrupt();
                    break;
                }
            }
        }
    }
}
